@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2016 Mark Hills <mark@xwax.org>
+ * Copyright (C) 2018 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -53,8 +53,7 @@ void index_init(struct index *ls)
 
 void index_clear(struct index *ls)
 {
-    if (ls->record != NULL)
-        free(ls->record);
+    free(ls->record); /* may be NULL */
 }
 
 /*
@@ -171,10 +170,19 @@ static int record_cmp_bpm(const struct record *a, const struct record *b)
 
 static bool record_match_word(struct record *re, const char *match)
 {
-    if (strcasestr(re->artist, match) != NULL)
-        return true;
-    if (strcasestr(re->title, match) != NULL)
-        return true;
+    /* Some records provide a dedicated string for matching against,
+     * in the same locale as "match" */
+
+    if (re->match) {
+        if (strcasestr(re->match, match) != NULL)
+            return true;
+    } else {
+        if (strcasestr(re->artist, match) != NULL)
+            return true;
+        if (strcasestr(re->title, match) != NULL)
+            return true;
+    }
+
     return false;
 }
 
